/*
ID: espr1t
LANG: C++
TASK: game
CONTEST: CodeIT, Round 3
KEYWORDS: NP, Game, Dynamic Programming
*/

#include <cstdio>
#include <cstring>
#include <queue>
#include <vector>
#include <algorithm>
using namespace std;

#define MAX 32
#define MAX_LEN 22
#define MAX_POSITIONS 10
#define MAX_EXEC_TIME 0.7

#define EMPTY -1
#define BLOCKED -2

struct Cell
{
	int row, col;
	Cell(int row_ = 0, int col_ = 0) : row(row_), col(col_) {}
};

struct State
{
	int row, col, mask;
};

int n, m;
int vis[MAX][MAX], okay[MAX][MAX];
int board[MAX][MAX], index[MAX][MAX];
int dir[4][2] = { {-1, 0}, {0, 1}, {1, 0}, {0, -1} };

unsigned sTime, cTime;
vector <Cell> ans, last;


double getTime()
{
	cTime = clock();
	return (double)(cTime - sTime) / CLOCKS_PER_SEC;
}

int validStart(int row, int col)
{
	Cell cur, nxt;
	queue <Cell> q;
	memset(vis, 0, sizeof(vis));

	int rem = board[row][col];
	q.push(Cell(row, col)); vis[row][col] = 1;
	
	last.clear();
	while (!q.empty())
	{
		Cell cur = q.front(); q.pop();
		last.push_back(cur);
		if (--rem <= 0) return 1;
		for (int i = 0; i < 4; i++)
		{
			nxt.row = cur.row + dir[i][0]; if (nxt.row < 0 || nxt.row >= n) continue;
			nxt.col = cur.col + dir[i][1]; if (nxt.col < 0 || nxt.col >= n) continue;
			if (okay[nxt.row][nxt.col] && !vis[nxt.row][nxt.col] && board[nxt.row][nxt.col] == EMPTY)
				{vis[nxt.row][nxt.col] = 1; q.push(nxt);}
		}
	}
	return 0;
}

char dyn[MAX][MAX][1 << MAX_POSITIONS];
State lst[MAX][MAX][1 << MAX_POSITIONS];

int coverAll(int row, int col)
{
	State cur, nxt;
	int numCells = board[row][col];
	
	queue <State> q;
	memset(dyn, -1, sizeof(dyn));
	nxt.row = row; nxt.col = col; nxt.mask = (1 << index[row][col]);
	q.push(nxt);
	dyn[nxt.row][nxt.col][nxt.mask] = 1;
	lst[nxt.row][nxt.col][nxt.mask].mask = -1;

	while (!q.empty())
	{
		cur = q.front(); q.pop();
		if (cur.mask == (1 << m) - 1) break;
		if (dyn[cur.row][cur.col][cur.mask] >= board[row][col])
			continue;

		for (int i = 0; i < 8; i++)
		{
			nxt.row = (i < 4 ? cur.row : row) + dir[i & 3][0]; if (nxt.row < 0 || nxt.row >= n) continue;
			nxt.col = (i < 4 ? cur.col : col) + dir[i & 3][1]; if (nxt.col < 0 || nxt.col >= n) continue;
			if (!okay[nxt.row][nxt.col] || board[nxt.row][nxt.col] != EMPTY) continue;
			nxt.mask = cur.mask;
			for (int c = 0; c < 4; c++)
			{
				int nrow = nxt.row + dir[c][0]; if (nrow < 0 || nrow >= n) continue;
				int ncol = nxt.col + dir[c][1]; if (ncol < 0 || ncol >= n) continue;
				if (index[nrow][ncol] != -1) nxt.mask |= (1 << index[nrow][ncol]);
			}
			if (dyn[nxt.row][nxt.col][nxt.mask] == -1)
			{
				dyn[nxt.row][nxt.col][nxt.mask] = dyn[cur.row][cur.col][cur.mask] + 1;
				lst[nxt.row][nxt.col][nxt.mask] = cur;
				q.push(nxt);
			}
		}
	}

	last.clear();
	if (cur.mask == (1 << m) - 1)
	{
		// Get cells
		queue <Cell> bfs;
		memset(vis, 0, sizeof(vis));
		
		vis[cur.row][cur.col] = 1;
		bfs.push(Cell(cur.row, cur.col));
		last.push_back(Cell(cur.row, cur.col));
		while (lst[cur.row][cur.col][cur.mask].mask != -1)
		{
			cur = lst[cur.row][cur.col][cur.mask];
			if (vis[cur.row][cur.col] == 0)
			{
				vis[cur.row][cur.col] = 1;
				bfs.push(Cell(cur.row, cur.col));
				last.push_back(Cell(cur.row, cur.col));
			}
		}
		Cell cCell, nCell;
		while (!bfs.empty() && (int)last.size() < board[row][col])
		{
			cCell = bfs.front(); bfs.pop();
			for (int i = 0; i < 4; i++)
			{
				nCell.row = cCell.row + dir[i][0]; if (nCell.row < 0 || nCell.row >= n) continue;
				nCell.col = cCell.col + dir[i][1]; if (nCell.col < 0 || nCell.col >= n) continue;
				if (!okay[nCell.row][nCell.col] || board[nCell.row][nCell.col] != EMPTY) continue;
				if (!vis[nCell.row][nCell.col])
				{
					bfs.push(nCell);
					vis[nCell.row][nCell.col] = 1;
					last.push_back(nCell);
					if ((int)last.size() >= board[row][col]) break;
				} 
			}
		}
		return 1;
	}
	return 0;
}


void eval()
{
	sTime = clock();

	// Find cells that are okay to be colored
	for (int row = 0; row < n; row++)
	{
		for (int col = 0; col < n; col++)
		{
			okay[row][col] = 1;
			for (int i = 0; i < 4; i++)
			{
				int nrow = row + dir[i][0]; if (nrow < 0 || nrow >= n) continue;
				int ncol = col + dir[i][1]; if (ncol < 0 || ncol >= n) continue;
				if (board[nrow][ncol] == BLOCKED) {okay[row][col] = 0; break;}
			}
		}
	}

	// Find all valid starting cells
	vector <Cell> starts;
	for (int row = 0; row < n; row++)
		for (int col = 0; col < n; col++)
			if (board[row][col] > 0 && okay[row][col])
				if (validStart(row, col)) starts.push_back(Cell(row, col));
	
	// Already lost?
	if (starts.size() == 0) return;

	// Sort cells by size using bubble sort, woohoo!
	for (int i = 0; i < (int)starts.size(); i++)
		for (int c = 0; c < (int)starts.size() - i - 1; c++)
			if (board[starts[c].row][starts[c].col] < board[starts[c + 1].row][starts[c + 1].col])
				swap(starts[c], starts[c + 1]);
	
	// Try to win in this move
	// DP from every cell trying to cover all other
	if (starts.size() <= MAX_POSITIONS)
	{
		// Index valid starts for DP
		m = (int)starts.size();
		memset(index, -1, sizeof(index));
		for (int i = 0; i < (int)starts.size(); i++)
			index[starts[i].row][starts[i].col] = i;
	
		for (int i = 0; i < (int)starts.size(); i++)
		{
			// Muse - Time Is Running Out
			if (getTime() > MAX_EXEC_TIME) break;

			// A little creepy, but...
			// One Cell to rule them all, One Cell to find them,
			// One Cell to bring them all and in the darkness bind them
			if (coverAll(starts[i].row, starts[i].col)) {ans = last; return;}
		}
	}

	// Too many to cover, or couldn't find a way to do so
	// Instead, choose cell with smallest number
	validStart(starts.back().row, starts.back().col);
	ans = last;
}

void readInput()
{
	FILE* in = fopen("game.in", "rt");
	fscanf(in, "%d", &n);
	char buff[MAX];
	for (int i = 0; i < n; i++)
	{
		for (int c = 0; c < n; c++)
		{
			fscanf(in, "%s", buff);
			if (buff[0] == 'B') board[i][c] = BLOCKED;
			else if (buff[0] == '#') board[i][c] = EMPTY;
			else sscanf(buff, "%d", &board[i][c]);
		}
	}
	fclose(in);
}

void writeOutput()
{
	FILE* out = fopen("game.out", "wt");
	fprintf(out, "%d\n", (int)ans.size());
	for (int i = 0; i < (int)ans.size(); i++)
		fprintf(out, "%d %d\n", ans[i].row, ans[i].col);
	fclose(out);
}

int main(void)
{
	readInput();
	eval();
	writeOutput();
	return 0;
}
